/*  Misfit Model 3D
 * 
 *  Copyright (c) 2004 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 *
 *  See the COPYING file for full license text.
 */


#include "cylindertoolwidget.h"

#include "3dmprefs.h"

#include <qlayout.h>
#include <qgroupbox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qslider.h>

CylinderToolWidget::CylinderToolWidget( Observer * observer, QWidget * parent )
   : QDockWindow ( QDockWindow::InDock, parent, "", WDestructiveClose ),
     m_observer( observer )
{
   const int DEFAULT_SMOOTH = 4;
   const int DEFAULT_WIDTH  = 100;
   const int DEFAULT_SCALE  = 100;
   const int DEFAULT_SHAPE  = 10;

   m_layout = boxLayout();

   m_smoothLabel = new QLabel( "Smooth", this, "" );
   m_layout->addWidget( m_smoothLabel );

   m_smoothValue = new QSpinBox( this, "" );
   m_layout->addWidget( m_smoothValue );

   m_smoothValue->setMinValue( 0 );
   m_smoothValue->setMaxValue( 15 );
   int smoothVal = DEFAULT_SMOOTH;
   if ( g_prefs.exists( "ui_cylindertool_smoothness" ) )
   {
      int val = g_prefs( "ui_cylindertool_smoothness" ).intValue();
      if ( val >= 0 && val <= 15 )
      {
         smoothVal = val;
      }
   }
   m_smoothValue->setValue( smoothVal );

   m_widthLabel = new QLabel( "Width", this, "" );
   m_layout->addWidget( m_widthLabel );

   m_widthValue = new QSpinBox( this, "" );
   m_layout->addWidget( m_widthValue );

   m_widthValue->setMinValue( 0 );
   m_widthValue->setMaxValue( 100 );
   int widthVal = DEFAULT_WIDTH;
   if ( g_prefs.exists( "ui_cylindertool_width" ) )
   {
      int val = g_prefs( "ui_cylindertool_width" ).intValue();
      if ( val >= 0 && val <= 100 )
      {
         widthVal = val;
      }
   }
   m_widthValue->setValue( widthVal );

   m_scaleLabel = new QLabel( "Scale", this, "" );
   m_layout->addWidget( m_scaleLabel );

   m_scaleValue = new QSpinBox( this, "" );
   m_layout->addWidget( m_scaleValue );

   m_scaleValue->setMinValue( 0 );
   m_scaleValue->setMaxValue( 100 );
   int scaleVal = DEFAULT_SCALE;
   if ( g_prefs.exists( "ui_cylindertool_scale" ) )
   {
      int val = g_prefs( "ui_cylindertool_scale" ).intValue();
      if ( val >= 0 && val <= 100 )
      {
         scaleVal = val;
      }
   }
   m_scaleValue->setValue( scaleVal );

   m_shapeLabel = new QLabel( "Shape", this, "" );
   m_layout->addWidget( m_shapeLabel );

   m_shapeValue = new QSlider( Qt::Horizontal, this, "" );
   m_layout->addWidget( m_shapeValue );

   m_shapeValue->setMinimumWidth( 64 );
   m_shapeValue->setMinValue( 0 );
   m_shapeValue->setMaxValue( DEFAULT_SHAPE * 2 );
   m_shapeValue->setValue( DEFAULT_SHAPE );
   m_shapeValue->setTickInterval( DEFAULT_SHAPE / 2);
   m_shapeValue->setTickmarks( QSlider::Below );

   connect( m_smoothValue, SIGNAL(valueChanged(int)), this, SLOT(smoothValueChanged(int)) );
   connect( m_widthValue,  SIGNAL(valueChanged(int)), this, SLOT(widthValueChanged(int))  );
   connect( m_scaleValue,  SIGNAL(valueChanged(int)), this, SLOT(scaleValueChanged(int))  );
   connect( m_shapeValue,  SIGNAL(valueChanged(int)), this, SLOT(shapeValueChanged(int))  );

   m_smoothLabel->show();
   m_smoothValue->show();
   m_widthLabel->show();
   m_widthValue->show();
   m_scaleLabel->show();
   m_scaleValue->show();
   //m_shapeLabel->hide();
   //m_shapeValue->hide();

   smoothValueChanged( smoothVal );
   widthValueChanged( widthVal );
   scaleValueChanged( scaleVal );
   shapeValueChanged( DEFAULT_SHAPE );
}

CylinderToolWidget::~CylinderToolWidget()
{
}

void CylinderToolWidget::smoothValueChanged( int newValue )
{
   g_prefs( "ui_cylindertool_smoothness" ) = newValue;
   m_observer->setSmoothValue( newValue );
}

void CylinderToolWidget::widthValueChanged( int newValue )
{
   g_prefs( "ui_cylindertool_width" ) = newValue;
   m_observer->setWidthValue( newValue );
}

void CylinderToolWidget::scaleValueChanged( int newValue )
{
   g_prefs( "ui_cylindertool_scale" ) = newValue;
   m_observer->setScaleValue( newValue );
}

void CylinderToolWidget::shapeValueChanged( int newValue )
{
   m_observer->setShapeValue( newValue );
}

